-- Mod:		AnywhereTrainer
-- Author:	DarthVon (thedpui02@sneakemail.com)
-- Version:	1.0-release
-- Date:	09/28/2008
-- ========================
-- Description:
--	Allows you to view your character advancement windows from anywhere.
--


AnywhereTrainer = {}

local DefaultCharacterWindow_UpdateMode = CharacterWindow.UpdateMode

------------------------------------
--
--
function AnywhereTrainer.Initialize( )

	AnywhereTrainer.TabData = {
		{ Name="AnywhereTrainerTopBookend", Template="AnywhereTrainerTopBookendTemplate", TextureWindow="", Slice="TabTopper", Anchor={ Point="topright", RelativeTo="CharacterWindow", RelativePoint="topleft", X=-43, Y=150 } },
		{ Name="AnywhereTrainerTabCareer", Template="AnywhereTrainerTabTemplate", TextureWindow="Icon", Slice="Tab-Core", Anchor={ Point="bottomleft", RelativeTo="AnywhereTrainerTopBookend", RelativePoint="topleft", X=0, Y=2 }, Tooltip=L"Career Training", OnClick=AnywhereTrainer.OnClickCareer },
		{ Name="AnywhereTrainerTabMastery", Template="AnywhereTrainerTabTemplate", TextureWindow="Icon", Slice="Tab-PassiveSkills", Anchor={ Point="bottomleft", RelativeTo="AnywhereTrainerTabCareer", RelativePoint="topleft", X=0, Y=2 }, Tooltip=L"Mastery Training", OnClick=AnywhereTrainer.OnClickMastery },
		{ Name="AnywhereTrainerTabRenown", Template="AnywhereTrainerTabTemplate", TextureWindow="Icon", Slice="Tab-RvR", Anchor={ Point="bottomleft", RelativeTo="AnywhereTrainerTabMastery", RelativePoint="topleft", X=0, Y=2 }, Tooltip=L"Renown Training", OnClick=AnywhereTrainer.OnClickRenown },
		{ Name="AnywhereTrainerTabTome", Template="AnywhereTrainerTabTemplate", TextureWindow="Icon", Slice="Tab-Tome", Anchor={ Point="bottomleft", RelativeTo="AnywhereTrainerTabRenown", RelativePoint="topleft", X=0, Y=2 }, Tooltip=L"Tome Training", OnClick=AnywhereTrainer.OnClickTome },
		{ Name="AnywhereTrainerBottomBookend", Template="AnywhereTrainerBottomBookendTemplate", TextureWindow="", Slice="TabBottom", Anchor={ Point="bottomleft", RelativeTo="AnywhereTrainerTabTome", RelativePoint="topleft", X=0, Y=2 } }
	}

	local w, h = WindowGetDimensions( "CharacterWindow" )
	WindowSetDimensions( "CharacterWindow", w + 43, h )

	AnywhereTrainer.ReadjustWindowAnchors( "CharacterWindowBackground", "bottomright", -43, 0 )

	for _, tab in ipairs(AnywhereTrainer.TabData) do
		CreateWindowFromTemplate( tab.Name, tab.Template, "CharacterWindow" )
		WindowAddAnchor( tab.Name, tab.Anchor.Point, tab.Anchor.RelativeTo, tab.Anchor.RelativePoint, tab.Anchor.X, tab.Anchor.Y )
		DynamicImageSetTextureSlice( tab.Name .. tab.TextureWindow, tab.Slice )
	end
	
	CharacterWindow.UpdateMode = AnywhereTrainer.CharacterWindow_UpdateMode

end


------------------------------------
--
--
function AnywhereTrainer.CharacterWindow_UpdateMode( mode )
	DefaultCharacterWindow_UpdateMode( mode )
	if( (mode == CharacterWindow.MODE_NORMAL) or (mode == CharacterWindow.MODE_DYE_MERCHANT) ) then
		local w, h = WindowGetDimensions( "CharacterWindow" )
		WindowSetDimensions( "CharacterWindow", w + 43, h )
	end
end


------------------------------------
--
--
function AnywhereTrainer.ReadjustWindowAnchors( windowName, point, xOffset, yOffset )

	local anchors = {}
	local anchorCount = WindowGetAnchorCount( windowName )
	for i=1,anchorCount do
		table.insert( anchors, { WindowGetAnchor( windowName, i ) } )
		if( anchors[i][1] == point ) then
			anchors[i][4] = anchors[i][4] + xOffset
			anchors[i][5] = anchors[i][5] + yOffset
		end
	end
	WindowClearAnchors( windowName )
	for i=1,anchorCount do
		WindowAddAnchor( windowName, anchors[i][1], anchors[i][3], anchors[i][2], anchors[i][4], anchors[i][5] )
	end

end


------------------------------------
--
--
function AnywhereTrainer.Shutdown( )

end


------------------------------------
--
--
function AnywhereTrainer.OnMouseOver( )
	for _, tab in ipairs(AnywhereTrainer.TabData) do
		if( tab.Name .. "InactiveImage" == SystemData.MouseOverWindow.name ) then
			if( tab.Tooltip ~= nil ) then
				Tooltips.CreateTextOnlyTooltip( SystemData.MouseOverWindow.name, tab.Tooltip )
				Tooltips.AnchorTooltip( Tooltips.ANCHOR_WINDOW_RIGHT );
			end
			return
		end
	end
end


------------------------------------
--
--
function AnywhereTrainer.OnLButtonUp( )
	for _, tab in ipairs(AnywhereTrainer.TabData) do
		if( tab.Name .. "InactiveImage" == SystemData.MouseOverWindow.name ) then
			if( tab.OnClick ~= nil ) then
				tab.OnClick( )
			end
			return
		end
	end
end


------------------------------------
--
--
function AnywhereTrainer.OnClickCareer( )
	if( not WindowGetShowing( "EA_Window_InteractionCoreTraining" ) ) then
		EA_Window_InteractionCoreTraining.Show( )
	else
		EA_Window_InteractionCoreTraining.Hide( )
	end
end


------------------------------------
--
--
function AnywhereTrainer.OnClickMastery( )
	if( not WindowGetShowing( "EA_Window_InteractionSpecialtyTraining" ) ) then
		EA_Window_InteractionSpecialtyTraining.Show( )
	else
		EA_Window_InteractionSpecialtyTraining.Hide( )
	end
end


------------------------------------
--
--
function AnywhereTrainer.OnClickRenown( )
	if( not WindowGetShowing( "EA_Window_InteractionRenownTraining" ) ) then
		EA_Window_InteractionRenownTraining.Show( )
	else
		EA_Window_InteractionRenownTraining.Hide( )
	end
end


------------------------------------
--
--
function AnywhereTrainer.OnClickTome( )
	if( not WindowGetShowing( "EA_Window_InteractionTomeTraining" ) ) then
		EA_Window_InteractionTomeTraining.Show( )
	else
		EA_Window_InteractionTomeTraining.Hide( )
	end
end