--[[
    Author: WarBagel, Immortal Sovereigns
--]]
 
ISHealBot = {}
 

-- settings constants
ISHealBot.SETTINGS_DEBUG = "Debug"
ISHealBot.SETTINGS_SPELLS = "Spells"
ISHealBot.SETTINGS_ENABLED = "Enabled"
ISHealBot.SETTINGS_STICKY_TARGET = "StickyTarget"
ISHealBot.SETTINGS_SHOW_MINIMAP = "ShowMinimap"

ISHealBot.MODIFIERS_ALT = "ALT"
ISHealBot.MODIFIERS_SHIFT = "SHIFT"
ISHealBot.MODIFIERS_CTRL = "CTRL"
ISHealBot.MODIFIERS_NONE = "NONE"

ISHealBot.SETTINGS_DEFAULTS = {
    [ISHealBot.SETTINGS_SPELLS] = {
    },
    [ISHealBot.SETTINGS_STICKY_TARGET] = true,
    [ISHealBot.SETTINGS_SHOW_MINIMAP] = true,
    [ISHealBot.SETTINGS_DEBUG] = false,
    [ISHealBot.SETTINGS_ENABLED] = true,
}

ISHealBot.MODIFIERS = {
    [ISHealBot.MODIFIERS_ALT] = tonumber(SystemData.ButtonFlags.ALT),
    [ISHealBot.MODIFIERS_SHIFT] = tonumber(SystemData.ButtonFlags.SHIFT),
    [ISHealBot.MODIFIERS_CTRL] = tonumber(SystemData.ButtonFlags.CONTROL),
    [ISHealBot.MODIFIERS_NONE] = 0,
}

ISHealBot.POSSIBLE_MODIFIERS = {
    [ISHealBot.MODIFIERS_ALT] = tonumber(SystemData.ButtonFlags.ALT),
    [ISHealBot.MODIFIERS_SHIFT] = tonumber(SystemData.ButtonFlags.SHIFT),
    [ISHealBot.MODIFIERS_CTRL] = tonumber(SystemData.ButtonFlags.CONTROL),
    [ISHealBot.MODIFIERS_NONE] = 0,
    [ISHealBot.MODIFIERS_ALT.."+"..ISHealBot.MODIFIERS_SHIFT] = (tonumber(SystemData.ButtonFlags.ALT)+tonumber(SystemData.ButtonFlags.SHIFT)),
    [ISHealBot.MODIFIERS_ALT.."+"..ISHealBot.MODIFIERS_CTRL] = (tonumber(SystemData.ButtonFlags.ALT)+tonumber(SystemData.ButtonFlags.CONTROL)),
    [ISHealBot.MODIFIERS_SHIFT.."+"..ISHealBot.MODIFIERS_CTRL] = (tonumber(SystemData.ButtonFlags.SHIFT)+tonumber(SystemData.ButtonFlags.CONTROL)),
    [ISHealBot.MODIFIERS_ALT.."+"..ISHealBot.MODIFIERS_CTRL.."+"..ISHealBot.MODIFIERS_SHIFT] = (tonumber(SystemData.ButtonFlags.ALT)+tonumber(SystemData.ButtonFlags.CONTROL)+tonumber(SystemData.ButtonFlags.SHIFT)),
}

-- constants
ISHealBot.WINDOW = "SquaredUnitFrame"
ISHealBot.WINDOW_SETTINGS = "ISHealBotSettingsWindow"
ISHealBot.WINDOW_SETTINGS_ASSIGNMENTS = "ISHealBotSettingsAssignmentWindow"
ISHealBot.WindowMinimapButton = "ISHealBotMapBtn"
ISHealBot.WindowShowMinimap = "ISHealBotSettingsShowMinimap"
ISHealBot.WindowStickyTarget = "ISHealBotSettingsStickyTarget"
ISHealBot.EMPTY_ICON = 168
ISHealBot.SEPARATOR = "_"
ISHealBot.Version = L"v2.2"

-- localized constants
ISHealBot.LOCAL_MINIMAP_BUTTON_MO = "MINIMAP_BUTTON_MOUSEOVER_TEXT"
ISHealBot.LOCAL_MINIMAP_MO = "MINIMAP_MOUSEOVER_TEXT"
ISHealBot.LOCAL_MINIMAP_OPT = "OPTION_SHOW_MINIMAP"
ISHealBot.LOCAL_STICKY_TARGET_OPT = "OPTION_STICKY_TARGET"
ISHealBot.LOCAL_STICKY_TARGET_MO = "STICKY_TARGET_MOUSEOVER_TEXT"
ISHealBot.LOCAL_INVALID_TARGET = "INVALID_TARGET"
ISHealBot.LOCAL_INIT_DEFAULTS = "INIT_DEFAULTS"
ISHealBot.LOCAL_INIT_ADDON = "INIT_ADDON"
ISHealBot.LOCAL_SETTINGS_TITLE = "SETTINGS_TITLE"
ISHealBot.LOCAL_MINIMAP_BUTTON_TITLE = "MINIMAP_BUTTON_TITLE"
ISHealBot.LOCAL_MINIMAP_BUTTON_DESC = "MINIMAP_BUTTON_DESC"
ISHealBot.LOCAL_SAVE = "SAVE"
ISHealBot.LOCAL_UNREGISTER_ERR = "UNREG_ERR"
ISHealBot.LOCAL_REGISTER_ERR = "REG_ERR"

ISHealBot.MESSAGES = {}

ISHealBot.MOUSEOVER_TEXT = {
      [ISHealBot.WindowMinimapButton] = ISHealBot.LOCAL_MINIMAP_BUTTON_MO,
      [ISHealBot.WindowShowMinimap] = ISHealBot.LOCAL_MINIMAP_MO,
      [ISHealBot.WindowStickyTarget] = ISHealBot.LOCAL_STICKY_TARGET_MO,
}

-- normal vars
ISHealBot.previousTarget = nil
ISHealBot.actions = {}
ISHealBot.Messages = {}


--[[
    Initializes the addon
--]]
function ISHealBot.Initialize()
    ISHealBot.InitLocale()
    ISHealBot.InitializeSettings()
    ISHealBot.RegisterSlashCommands()
    ISHealBot.CreateSettingsWindow()
    ISHealBot.SetSettingsVisible(false)
    ISHealBot.PrintSettings()

    if ISHealBot.GetSetting(ISHealBot.SETTINGS_ENABLED) then
        ISHealBot.Enable()
    end
end


function ISHealBot.InitLocale()
	local switch = {
		[SystemData.Settings.Language.ENGLISH] = L"enUS",
		[SystemData.Settings.Language.FRENCH] = L"frFR",
		[SystemData.Settings.Language.GERMAN] = L"deDE",
		[SystemData.Settings.Language.ITALIAN] = L"itIT",
		[SystemData.Settings.Language.SPANISH] = L"esES",
		[SystemData.Settings.Language.KOREAN] = L"koKR",
		[SystemData.Settings.Language.S_CHINESE] = L"zhCN",
		[SystemData.Settings.Language.T_CHINESE] = L"zhTW",
		[SystemData.Settings.Language.JAPANESE] = L"jaJP",
		[SystemData.Settings.Language.RUSSIAN] = L"ruRU",
	}
		
	local res = switch[SystemData.Settings.Language.active] 
	if not res then
		res = L"enUS"
	end

    ISHealBot.Messages = ISHealBot.MESSAGES[res]
end


function ISHealBot.GetMessage(key)
    return ISHealBot.Messages[key]
end


--[[
    Shutsdown the addon
--]]
function ISHealBot.Shutdown()
    ISHealBot.Disable()
end


--[[
    Disables the addon
--]]
function ISHealBot.Disable()
    -- return squared to previous state
    if Squared then
        -- unregister with squared
        Squared.UnregisterEventHandler("unitlclick", ISHealBot.UnitLClick)
        Squared.UnregisterEventHandler("unitrclick", ISHealBot.UnitRClick)
        -- hide action buttons
        --ISHealBot.SetActionButtonVisibility(false)
    else
        ISHealBot.Print(ISHealBot.GetMessage(ISHealBot.LOCAL_UNREGISTER_ERR))
    end
    -- unregister with event handlers
    UnregisterEventHandler(SystemData.Events.PLAYER_END_CAST, "ISHealBot.OnPlayerEndCast")
    UnregisterEventHandler(SystemData.Events.SPELL_CAST_CANCEL, "ISHealBot.OnPlayerEndCast")
    UnregisterEventHandler(SystemData.Events.INTERACT_DONE, "ISHealBot.OnPlayerEndCast")
end


--[[
    Enables the addon
--]]
function ISHealBot.Enable()
    -- setup squared
    if Squared then
        -- register with squared
        Squared.RegisterEventHandler("unitlclick", ISHealBot.UnitLClick)
        Squared.RegisterEventHandler("unitrclick", ISHealBot.UnitRClick)
        -- show action buttons
        --ISHealBot.SetActionButtonVisibility(true)
    else
        ISHealBot.Print(ISHealBot.GetMessage(ISHealBot.LOCAL_REGISTER_ERR))
    end
    -- register with event handlers
    RegisterEventHandler(SystemData.Events.PLAYER_END_CAST, "ISHealBot.OnPlayerEndCast")
    RegisterEventHandler(SystemData.Events.SPELL_CAST_CANCEL, "ISHealBot.OnPlayerEndCast")
    RegisterEventHandler(SystemData.Events.INTERACT_DONE, "ISHealBot.OnPlayerEndCast")
end


--[[
    Sets the visibility of all the squared action buttons
--]]
function ISHealBot.SetActionButtonVisibility(isVisible)
    for g = 1, 10 do
        for m = 1, 6 do
            local window = "SquaredUnit_"..g.."_"..m.."Action"
            if DoesWindowExist(window) then
                WindowSetShowing(window, isVisible)
            end
        end
    end
end


--[[
    Verify Squared is installed, returns true if it is, otw false
--]]
function ISHealBot.VerifyState()
    return (Squared ~= nil)
end


--[[
    If the previous target is set - reset the current friendly target to it
--]]
function ISHealBot.OnPlayerEndCast(spellId, isChannel, castTime)
    -- if we are disabled, exit w/o doing anyting
    if not ISHealBot.GetSetting(ISHealBot.SETTINGS_ENABLED) then
        return
    end
    -- if sticky targeting is disabled, exit
    if not ISHealBot.GetSetting(ISHealBot.SETTINGS_STICKY_TARGET) then
        return
    end
    -- if we are valid, continue
    if ISHealBot.VerifyState() then
        if ISHealBot.previousTarget and ISHealBot.previousTarget ~= L"" then
            -- target previous target
            if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                ISHealBot.Print("Restoring previous target")
            end
            TargetPlayer(ISHealBot.previousTarget)
            ISHealBot.previousTarget = nil
        else
            if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                ISHealBot.Print("There is no previous target, will not restore")
            end
        end
    end
end
 

--[[
    Targets the player for the current unit frame if the shift modifier is used
--]]
function ISHealBot.UnitRClick(group, member, flags)
    -- if we are disabled, exit w/o doing anyting
    if not ISHealBot.GetSetting(ISHealBot.SETTINGS_ENABLED) then
        return false
    end
    if ISHealBot.VerifyState() then
        if group and member then
            local modifier = flags
            if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                ISHealBot.Print("Modifier="..flags)
            end
            if modifier == ISHealBot.MODIFIERS[ISHealBot.MODIFIERS_SHIFT] then
                local unit = Squared.GetUnit(group, member)
                local target = nil
                if unit then
                    target = unit.name
                    -- target player
                    if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                        ISHealBot.Print(L"Targeting "..target)
                    end
                    TargetPlayer(target)
                    return true
                end
            else
                if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                    local message = "Doing Nothing. Missing Target"
                    ISHealBot.Print(message)
                end
            end
        else
            if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                local message = "Doing Nothing. "
                if not group then
                    message = message.."Missing Group. "
                end
                if not member then
                    message = message.."Missing Member."
                end
                ISHealBot.Print(message)
            end
        end
    end

    return false
end


--[[
    Targets the existing player and sets the previous target (if available)
--]]
function ISHealBot.HandleTarget(target, spellId)
    -- get current target so we can restore it
    local originalTarget = TargetInfo:UnitName("selffriendlytarget")
    -- if we are using a sticky target, set it
    if ISHealBot.GetSetting(ISHealBot.SETTINGS_STICKY_TARGET) then
        if TargetInfo:UnitIsNPC("selffriendlytarget") then
            ISHealBot.previousTarget = nil
        else
            ISHealBot.previousTarget = originalTarget
        end
    end
    -- target player
    if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
        ISHealBot.Print(L"Targeting "..target)
    end
    TargetPlayer(target)
    -- is this a valid target - if not, reset
    TargetInfo:UpdateFromClient()
    if spellId and not IsTargetValid(spellId) then
        if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
            ISHealBot.Print(L"Target is not valid. Resetting to "..originalTarget)
        end
        AlertTextWindow.AddLine(SystemData.AlertText.Types.COMBAT, ISHealBot.GetMessage(ISHealBot.LOCAL_INVALID_TARGET))
        TargetPlayer(originalTarget)
        return false
    end
    return true
end


function ISHealBot.ArePlayerNamesEqual(p1, p2)
    if p1 and p2 then
        local p1N = p1:match(L"([^^]+)^?([^^]*)")
        local p2N = p2:match(L"([^^]+)^?([^^]*)")
        return p1N == p2N
    elseif not p1 and not p2 then
        return true
    end

    return false
end


--[[
    Set ability and target
--]]
function ISHealBot.UnitLClick(group, member, flags)
    -- if we are disabled, exit w/o doing anyting
    if not ISHealBot.GetSetting(ISHealBot.SETTINGS_ENABLED) then
        return false
    end
    if ISHealBot.VerifyState() then
        local modifier = flags
        if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
            ISHealBot.Print("Modifier="..flags)
        end
        local button = "SquaredUnit_"..group.."_"..member.."Action"
        local spellId = ISHealBot.GetSpellByModifier(modifier)
        local unit = Squared.GetUnit(group, member)
        local target = nil
        if unit then
            target = unit.name
        end
        if button and target then
            if spellId then
                -- handle target
                if ISHealBot.HandleTarget(target, spellId) then
                    -- assign correct ability
                    if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                        ISHealBot.Print("Assigning ability "..spellId.." to "..button)
                    end
                    ISHealBot.AssignAbility(spellId, button)

                else
                    -- clear the ability
                    if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                        ISHealBot.Print("Clear ability on "..button)
                    end
                    ISHealBot.AssignAbility(0, button)
                end
            else
                -- clear the ability
                if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                    ISHealBot.Print("Clear ability on "..button)
                end
                ISHealBot.AssignAbility(0, button)
            end
        else
            if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                local message = "Doing Nothing. "
                if not spellId then
                    message = message.."Missing Spell Id. "
                end
                if not target then
                    message = message.."Missing Target. "
                end
                if not button then
                    message = message.."Missing Button."
                end
                ISHealBot.Print(message)
            end
        end
    end
end


--[[
    Assigns the button the specified ability
--]]
function ISHealBot.AssignAbility(actionId, window)
    local actionType = 0
    if actionId > 0 then
        actionType = GameData.PlayerActions.DO_ABILITY
    end
    WindowSetGameActionTrigger (window, actionId)
    WindowSetGameActionData (window, actionType, actionId, L"")
end


--[[
    Returns the ability id of the spell associated with the modifier combination
--]]
function ISHealBot.GetSpellByModifier(modifier)
    if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
        if modifier then
            ISHealBot.Print("Is Using Modifier - "..modifier)
        else
            ISHealBot.Print("Is Not Using Modifier.")
        end
    end
    local spellId = nil
    local spells = ISHealBot.GetSetting(ISHealBot.SETTINGS_SPELLS)
    if spells then
        spellId = spells[modifier]
    end
    return spellId
end


--[[
    Prints messages to console
--]]
function ISHealBot.Print(msg)
    EA_ChatWindow.Print(towstring(msg))
end


--[[
    Sets specified option to the specified value (cannot be nil)
--]]
function ISHealBot.SetSetting(key, value)
    local playerSettings = ISHealBotSettings[WStringToString(GameData.Player.name)]
    playerSettings[key] = value
end


--[[
    gets the value for the specified option, return nil if missing
--]]
function ISHealBot.GetSetting(key)
    local playerSettings = ISHealBotSettings[WStringToString(GameData.Player.name)]
    return playerSettings[key]
end


--[[
    Sets player defaults
--]]
function ISHealBot.SetPlayerDefaults()
    ISHealBotSettings[WStringToString(GameData.Player.name)] = {}
    ISHealBot.VerifyAllSettingsExist()
end

--[[
    gets the value for the specified option, return nil if missing
--]]
function ISHealBot.GetSetting(key)
    local playerSettings = ISHealBotSettings[WStringToString(GameData.Player.name)]
    return playerSettings[key]
end


--[[
    Initializes all settings for the first time
--]]
function ISHealBot.CreateSettings()
    ISHealBotSettings = {}
end


--[[
    Verifies that all the settings exist, if they do not, it will add the missing ones
    assigning them the default values
--]]
function ISHealBot.VerifyAllSettingsExist()
    for k,v in pairs(ISHealBot.SETTINGS_DEFAULTS) do
        if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
            ISHealBot.Print("Verifying setting "..k)
        end
        local foundSetting = false
        
        -- search player settings for setting
        for k1,v1 in pairs(ISHealBotSettings[WStringToString(GameData.Player.name)]) do
            if k1 == k then
                foundSetting = true
            end
        end
        -- if we didnt find it - add the default
        if not foundSetting then
            if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                local value = v
                if type(value)=="boolean" then
                    if value then
                        value = "true"
                    else
                        value = "false"
                    end
                end
                ISHealBot.Print("Creating setting key="..k..", value="..value)
            end
            ISHealBot.SetSetting(k, v)
        else
            if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
                local value = ISHealBot.GetSetting(k)
                if type(value)=="boolean" then
                    if value then
                        value = "true"
                    else
                        value = "false"
                    end
                end
                if type(value)~="table" then
                    ISHealBot.Print("Found setting key="..k..", value="..value)
                else
                    ISHealBot.Print("Found setting key="..k..", value=table")
                end
            end
        end
    end
end


--[[
    Loads settings
--]]
function ISHealBot.InitializeSettings()
    -- create settings if we have none
    if not ISHealBotSettings then
        ISHealBot.CreateSettings()
    end
    -- load player settings
    if not ISHealBotSettings[WStringToString(GameData.Player.name)] then
        -- we have none - create the player settings
        ISHealBot.Print(ISHealBot.GetMessage(ISHealBot.LOCAL_INIT_DEFAULTS) .. GameData.Player.name .. L"...")
        ISHealBot.SetPlayerDefaults()
    else
        -- we have player settings - print out current settings
        ISHealBot.VerifyAllSettingsExist()
        ISHealBot.Print(ISHealBot.GetMessage(ISHealBot.LOCAL_INIT_ADDON)..L"...")
    end
end


--[[
    Shows/hides the main window
--]]
function ISHealBot.SetSettingsVisible(visible)
    if visible then
        ISHealBot.LoadSettings()
    end
    WindowSetShowing(ISHealBot.WINDOW_SETTINGS, visible)
end


--[[
    Creates the settings window
--]]
function ISHealBot.CreateSettingsWindow()
    CreateWindow(ISHealBot.WindowMinimapButton, true)
    LayoutEditor.RegisterWindow(ISHealBot.WindowMinimapButton, ISHealBot.GetMessage(ISHealBot.LOCAL_MINIMAP_BUTTON_TITLE), ISHealBot.GetMessage(ISHealBot.LOCAL_MINIMAP_BUTTON_DESC), true, true, true, nil)
    ISHealBot.SetMinimapVisible(ISHealBot.GetSetting(ISHealBot.SETTINGS_SHOW_MINIMAP))
    CreateWindow(ISHealBot.WINDOW_SETTINGS, false)
    LabelSetText(ISHealBot.WINDOW_SETTINGS.."TitleBarText", ISHealBot.GetMessage(ISHealBot.LOCAL_SETTINGS_TITLE)..L" "..ISHealBot.Version)
    ButtonSetText(ISHealBot.WINDOW_SETTINGS.."_ControlPanelAccept", ISHealBot.GetMessage(ISHealBot.LOCAL_SAVE))
    local possible = ISHealBot.POSSIBLE_MODIFIERS
    local relative = nil
    local parent = ISHealBot.WINDOW_SETTINGS_ASSIGNMENTS
    for k, v in pairs(possible) do
        local window = ISHealBot.GetAbilityAssignmentWindow(v)
        if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then
            ISHealBot.Print("Creating "..window)
        end
        CreateWindowFromTemplate(window, "ISHealBotSettingsAssignmentTemplate", parent)
        if relative then
            WindowAddAnchor(window, "bottom", relative, "top", 0, 0)
        else
            WindowAddAnchor(window, "topleft", parent, "topleft", 0, 0)
        end
        LabelSetText(window.."_Mod", ISHealBot.GetMessage(k))
        local texture, x, y = GetIconData(ISHealBot.EMPTY_ICON)
        if texture then
            DynamicImageSetTexture(window.."_Icon", texture, x, y)
        end
        relative = window
    end
    LabelSetText(ISHealBot.WindowShowMinimap.."Label", ISHealBot.GetMessage(ISHealBot.LOCAL_MINIMAP_OPT)) 
    ButtonSetCheckButtonFlag(ISHealBot.WindowShowMinimap, true)
    LabelSetText(ISHealBot.WindowStickyTarget.."Label", ISHealBot.GetMessage(ISHealBot.LOCAL_STICKY_TARGET_OPT))
    ButtonSetCheckButtonFlag(ISHealBot.WindowStickyTarget, true)
end


--[[
    Returns the window name for the given modifier
--]]
function ISHealBot.GetAbilityAssignmentWindow(modifier)
    return ISHealBot.WINDOW_SETTINGS_ASSIGNMENTS..ISHealBot.SEPARATOR..modifier
end


--[[
    Loads the settings into the window
--]]
function ISHealBot.LoadSettings()
    local assigned = ISHealBot.GetSetting(ISHealBot.SETTINGS_SPELLS)
    local possible = ISHealBot.POSSIBLE_MODIFIERS
    -- loop thru possibles, determining if we have any values for them
    for k, v in pairs(possible) do
        -- do we have an assignment
        local spellId = assigned[v]
        local modifier = k
        local modifierValue = v
        -- populate record in table
        ISHealBot.AssignSettingsAbility(modifierValue, modifier, spellId)
    end
    local showMinimap = ISHealBot.GetSetting(ISHealBot.SETTINGS_SHOW_MINIMAP)
    if showMinimap == nil then showMinimap = false end
    ButtonSetPressedFlag(ISHealBot.WindowShowMinimap, showMinimap)
    local stickyTarget = ISHealBot.GetSetting(ISHealBot.SETTINGS_STICKY_TARGET)
    if stickyTarget == nil then stickyTarget = false end
    ButtonSetPressedFlag(ISHealBot.WindowStickyTarget, stickyTarget)
end


--[[
    Updates the settings window the specified ability/monitor assignment
--]]
function ISHealBot.AssignSettingsAbility(modifierValue, modifier, actionId)
    local iconId = 0
    local name = L""
    if actionId or actionId == 0 then
        local abilityData = GetAbilityData(actionId)
        iconId = abilityData.iconNum
        name = abilityData.name
        ISHealBot.actions[modifierValue] = actionId
    end
    local window = ISHealBot.GetAbilityAssignmentWindow(modifierValue)
    local iconWindow = window.."_Icon"
    local labelWindow = window.."_Text"
    local texture, x, y = GetIconData(iconId)
    DynamicImageSetTexture(iconWindow, texture, x, y)
    LabelSetText(labelWindow, name)
end



--[[
    Saves the settings into the window
--]]
function ISHealBot.SaveSettings()
    local spells = ISHealBot.GetSetting(ISHealBot.SETTINGS_SPELLS)
    -- clear spells
    for k, v in pairs(spells) do
        table.remove(spells)
    end
    -- reassign spells
    for m, s in pairs(ISHealBot.actions) do
        -- if we have a valid assignment, save
        if s >= 0  then
            spells[m] = s
        end
    end
    local showMinimap = ButtonGetPressedFlag(ISHealBot.WindowShowMinimap)
    if showMinimap == nil then showMinimap = false end
    ISHealBot.SetSetting(ISHealBot.SETTINGS_SHOW_MINIMAP, showMinimap)
    ISHealBot.SetMinimapVisible(showMinimap)
    local stickyTarget = ButtonGetPressedFlag(ISHealBot.WindowStickyTarget)
    if stickyTarget == nil then stickyTarget = false end
    ISHealBot.SetSetting(ISHealBot.SETTINGS_STICKY_TARGET, stickyTarget)
end


--[[
    Handles the drag-n-drop functionality for assigning abilities to the settings window
--]]
function ISHealBot.AbilityCursorSwap (flags, x, y)
    if (Cursor.IconOnCursor ()) then
        -- Only existing abilities that are not tactics can be dropped
        local abilityData = Player.GetAbilityData (Cursor.Data.ObjectId)
        if (abilityData == nil) then
            return
        end
        if (abilityData.abilityType == GameData.AbilityType.TACTIC) then
            return ActionButtonAlert (StringTables.Default.TEXT_TACTIC_DROP_ERROR)
        end
        -- assign cursor id as ability id
        local spellId = Cursor.Data.ObjectId
        Cursor.Clear ()
        -- get modifier
        local modifier = SystemData.MouseOverWindow.name:match(ISHealBot.WINDOW_SETTINGS_ASSIGNMENTS..ISHealBot.SEPARATOR.."([0-9]+)")
        if modifier then
            modifier = tonumber(modifier)
        end
        -- assign ability
        if modifier then
            local possible = ISHealBot.POSSIBLE_MODIFIERS
            for k,v in pairs(possible) do
                if v == modifier then
                    ISHealBot.AssignSettingsAbility(modifier, k, spellId)
                    return
                end
            end
        end
    end
end


--[[
    Clears the given ability in the settings window
--]]
function ISHealBot.AbilityClear()
    -- get modifier
    local modifier = SystemData.MouseOverWindow.name:match(ISHealBot.WINDOW_SETTINGS_ASSIGNMENTS..ISHealBot.SEPARATOR.."([0-9]+)")
    if modifier then
        modifier = tonumber(modifier)
    end
    -- assign ability
    if modifier then
        local possible = ISHealBot.POSSIBLE_MODIFIERS
        for k,v in pairs(possible) do
            if v == modifier then
                ISHealBot.AssignSettingsAbility(modifier, k, 0)
                return
            end
        end
    end
end


--[[
    Applies the changes in the settings window
--]]
function ISHealBot.AcceptSettingsChanges()
    ISHealBot.SaveSettings()
    ISHealBot.SetSettingsVisible(false)
end


--[[
    Cancels the changes in the settings window
--]]
function ISHealBot.CancelSettingsChanges()
    ISHealBot.SetSettingsVisible(false)
end


--[[
    Shows the settings window
--]]
function ISHealBot.ShowOptions()
    ISHealBot.SetSettingsVisible(true)
end


function ISHealBot.HandleSlashCommand(args)
    if args then
        local opt, val = args:lower():match("([a-z0-9]+)[ ]?(.*)")
        if opt == "set" then
            local opt2, val2 = val:match("([a-z0-9]+)[ ]?(.*)")
            if opt2 == "stickytarget" then
                if val2 == "on" then
                    ISHealBot.SetSetting(ISHealBot.SETTINGS_STICKY_TARGET, true)
                    ISHealBot.Print("ISHealBot sticky targeting is now On")
                elseif val2 == "off" then
                    ISHealBot.SetSetting(ISHealBot.SETTINGS_STICKY_TARGET, false)
                    ISHealBot.Print("ISHealBot sticky targeting is now Off")
                else
                    ISHealBot.Print("Please choose to set sticky targeting on or off")
                end
            elseif opt2 == "debug" then
                if val2 == "on" then
                    ISHealBot.SetSetting(ISHealBot.SETTINGS_DEBUG, true)
                    ISHealBot.Print("ISHealBot Debug is now On")
                elseif val2 == "off" then
                    ISHealBot.SetSetting(ISHealBot.SETTINGS_DEBUG, false)
                    ISHealBot.Print("ISHealBot Debug is now Off")
                else
                    ISHealBot.Print("Please choose to set Debug on or off")
                end
            elseif opt2 == "showminimap" then
                if val2 == "on" then
                    ISHealBot.SetSetting(ISHealBot.SETTINGS_SHOW_MINIMAP, true)
                    ISHealBot.SetMinimapVisible(true)
                    ISHealBot.Print("ISHealBot ShowMinimap is now On")
                elseif val2 == "off" then
                    ISHealBot.SetSetting(ISHealBot.SETTINGS_SHOW_MINIMAP, false)
                    ISHealBot.SetMinimapVisible(false)
                    ISHealBot.Print("ISHealBot ShowMinimap is now Off")
                else
                    ISHealBot.Print("Please choose to set Debug on or off")
                end
            else
                ISHealBot.Print("Usage")
                ISHealBot.Print("/ishealbot options")
                ISHealBot.Print("/ishealbot set stickytarget on|off")
                ISHealBot.Print("/ishealbot set debug on|off")
                ISHealBot.Print("/ishealbot on|off")
            end
        elseif opt == "options" then
            ISHealBot.ShowOptions()
        elseif opt == "on" then
            if not ISHealBot.GetSetting(ISHealBot.SETTINGS_ENABLED) then
                ISHealBot.SetSetting(ISHealBot.SETTINGS_ENABLED, true)
                ISHealBot.Enable()
                ISHealBot.Print("ISHealBot is now On")
            else
                ISHealBot.Print("ISHealBot is already On. Nothing has changed.")
            end
        elseif opt == "off" then
            if ISHealBot.GetSetting(ISHealBot.SETTINGS_ENABLED) then
                ISHealBot.SetSetting(ISHealBot.SETTINGS_ENABLED, false)
                ISHealBot.Disable()
                ISHealBot.Print("ISHealBot is now Off")
            else
                ISHealBot.Print("ISHealBot is already Off. Nothing has changed.")
            end
        elseif opt == "print" then
            ISHealBot.PrintSettings()
        else
            ISHealBot.Print("Usage")
            ISHealBot.Print("/ishealbot options")
            ISHealBot.Print("/ishealbot set stickytarget on|off")
            ISHealBot.Print("/ishealbot set debug on|off")
            ISHealBot.Print("/ishealbot on|off")
        end
    end
end


--[[
    Registers addon with LibSlash
--]]
function ISHealBot.RegisterSlashCommands()
    if LibSlash then
        LibSlash.RegisterSlashCmd("ishealbot", ISHealBot.HandleSlashCommand)
        LibSlash.RegisterSlashCmd("ishb", ISHealBot.HandleSlashCommand)
    else
        ISHealBot.Print("LibSlash is missing, please install it.")
    end
end


--[[
    Shows/hides the minimap button
--]]
function ISHealBot.SetMinimapVisible(visible)
    WindowSetShowing(ISHealBot.WindowMinimapButton, visible)
end


--[[
    Prints out the current settings
--]]
function ISHealBot.PrintSettings()
    -- we have player settings - print out current settings
    local optDebug = "false"
    if ISHealBot.GetSetting(ISHealBot.SETTINGS_DEBUG) then optDebug = "true" end
    local optEnabled = "false"
    if ISHealBot.GetSetting(ISHealBot.SETTINGS_ENABLED) then optEnabled = "true" end
    local optStickyTargeting = "false"
    if ISHealBot.GetSetting(ISHealBot.SETTINGS_STICKY_TARGET) then optStickyTargeting = "true" end
    local optShowMinimap = "false"
    if ISHealBot.GetSetting(ISHealBot.SETTINGS_SHOW_MINIMAP) then optShowMinimap = "true" end
    ISHealBot.Print("ISHealBot Options:")
    ISHealBot.Print("Enabled=" .. optEnabled)
    ISHealBot.Print("StickyTarget=" .. optStickyTargeting)
    ISHealBot.Print("ShowMinimap=" .. optShowMinimap)
    ISHealBot.Print("Debug=" .. optDebug)
end
        

--[[
 Shows tooltip for map button
--]]
function ISHealBot.OnMouseover()
    local window = SystemData.ActiveWindow.name
    Tooltips.CreateTextOnlyTooltip( window, nil )
    
    local row = 1
    local column = 1
    Tooltips.SetTooltipText( row, column, ISHealBot.GetMessage(ISHealBot.MOUSEOVER_TEXT[window]))
    
    Tooltips.Finalize()
    Tooltips.AnchorTooltip( Tooltips.ANCHOR_WINDOW_LEFT)
end
